#!/usr/bin/env python
"""test program to exercise multicast send/receive."""

from __future__ import print_function


import sys, time
from socket import socket, AF_INET, SOCK_DGRAM, IPPROTO_UDP, SOL_SOCKET, \
    SO_REUSEADDR, IPPROTO_IP, IP_ADD_MEMBERSHIP, IP_MULTICAST_TTL, IP_MULTICAST_LOOP,   \
    inet_aton, INADDR_ANY
from struct import pack

from golang import b, u
from golang import sync, context


group4 = '224.1.1.1'
group6 = 'XXX'
port   =  5678

# mjoin_tx returns socket prepated to send/receive to/from multicast group:port.
def mjoin_tx(group, port, ttl=2):
    # XXX autodetect ip4/ip6 based on group addr
    sk = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)

    # set ttl and default destination address (for tx)
    sk.setsockopt(IPPROTO_IP, IP_MULTICAST_TTL, pack('b', ttl))
    sk.connect((group, port))

    # so that what we send is received locally as well
    sk.setsockopt(IPPROTO_IP, IP_MULTICAST_LOOP, 1)
    return sk

# mjoin_rx returns socket prepated to send/receive to/from multicast group:port.
def mjoin_rx(group, port):
    # XXX autodetect ip4/ip6 based on group addr
    sk = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)

    sk.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    sk.bind((group, port))

    # join the group (for rx)
    mreq = pack("4sl", inet_aton(group), INADDR_ANY)
    sk.setsockopt(IPPROTO_IP, IP_ADD_MEMBERSHIP, mreq)

    return sk


def txloop(ctx, sk, pkt):
    i = 0
    while 1:
        if ctx.err():
            raise ctx.err()
        i += 1
        pkt_ = pkt + (b'.%d' % i)
        print("tx: %s ..." % u(pkt_))
        sk.send(pkt_)
        time.sleep(1)

def rxloop(ctx, sk):
    while 1:
        if ctx.err():
            raise ctx.err()
        pkt = sk.recv(4096)
        print("rx: %s" % u(pkt))


def main():
    G = (group4, port)
    action = sys.argv[1]

    wg = sync.WorkGroup(context.background())
    if "tx" in action:
        sktx = mjoin_tx(*G)
        wg.go(txloop, sktx, b(sys.argv[2]))
    if "rx" in action:
        skrx = mjoin_rx(*G)
        wg.go(rxloop, skrx)

    wg.wait()

if __name__ == '__main__':
    main()
